@@ -7,6 +7,13 @@ def SQLFormatWithPrefix(p_szCommentSQLScript):
     # 把所有的SQL换行, 第一行加入[SQL >]， 随后加入[   >]
     m_FormattedString = None
     m_CommentSQLLists = p_szCommentSQLScript.split('\n')
+    # 从最后一行开始，依次删除空行，空行内容不打印
+    for m_nPos in range(len(m_CommentSQLLists), 0, -1):
+        if len(m_CommentSQLLists[m_nPos-1].strip()) == 0:
+            del m_CommentSQLLists[-1]
+        else:
+            break
+    # 拼接字符串
     for m_nPos in range(0, len(m_CommentSQLLists)):
         if m_nPos == 0:
             m_FormattedString = 'SQL> ' + m_CommentSQLLists[m_nPos]
@@ -68,13 +75,13 @@ def SQLAnalyze(p_SQLCommandPlainText):
                 # 当前已经在注释中，需要找到*/ 来标记注释的结束
                 m_nBlockCommentsEnd = SQLCommands[m_nPos].find('*/')
                 if m_nBlockCommentsEnd != -1:
-                    # 注释已经结束
+                    # 注释已经结束， 注释部分替换为空格
                     SQLCommands[m_nPos] = re.sub(r'.', ' ', SQLCommands[m_nPos][0:m_nBlockCommentsEnd + 2]) + \
                                           SQLCommands[m_nPos][m_nBlockCommentsEnd + 2:]
                     m_bInCommentBlock = False
                     continue
                 else:
-                    # 本行完全还在注释中
+                    # 本行完全还在注释中，全部替换为空格
                     SQLCommands[m_nPos] = re.sub(r'.', ' ', SQLCommands[m_nPos])
                     break
 
@@ -94,47 +101,62 @@ def SQLAnalyze(p_SQLCommandPlainText):
                         SQLCommands[m_nPos], re.IGNORECASE):
                 m_SQL = SQLCommands[m_nPos].strip()
                 SQLSplitResults.append(m_SQL)
-                m_CommentSQL = SQLCommandsWithComments[m_nPos]
+                SQLSplitResultsWithComments.append(SQLCommandsWithComments[m_nPos])
                 if m_nPos == len(SQLCommands) - 1:
                     # 如果本行就是最后一行
-                    SQLSplitResultsWithComments.append(m_CommentSQL)
                     m_NewSQLWithCommentsLastPos = 0
                 else:
-                    # 一直找到下一个有效语句的开始
+                    # 从当前语句开始，一直找到下一个有效语句的开始，中间全部是注释
+                    m_CommentSQL = None
                     for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
                         if len(SQLCommands[m_CommentPos].lstrip()) != 0:
                             # 这是一个非完全空行，有效内容从第一个字符开始
                             nLeadSpace = len(SQLCommands[m_CommentPos]) - len(SQLCommands[m_CommentPos].lstrip())
-                            m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
                             m_NewSQLWithCommentsLastPos = nLeadSpace
-                            SQLSplitResultsWithComments.append(m_CommentSQL)
                             break
                         else:
                             # 完全注释行
-                            m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                    SQLSplitResults.append("")
+                    SQLSplitResultsWithComments.append(m_CommentSQL)
                 continue
 
-            # 如果本行只有唯一的内容，就是段落终止符的话，本语句没有任何意义
+            # 如果本行只有唯一的内容，就是段落终止符的话，本语句没有意义，不会执行；但是注释有意义
+            # / 不能送给SQL解析器
             if re.match(r'^/$', SQLCommands[m_nPos]):
                 SQLSplitResults.append("")
-                m_CommentSQL = SQLCommandsWithComments[m_nPos]
+                SQLSplitResultsWithComments.append(SQLCommandsWithComments[m_nPos])
                 if m_nPos == len(SQLCommands) - 1:
                     # 如果本行就是最后一行
-                    SQLSplitResultsWithComments.append(m_CommentSQL)
                     m_NewSQLWithCommentsLastPos = 0
                 else:
-                    # 一直找到下一个有效语句的开始
+                    # 从当前语句开始，一直找到下一个有效语句的开始，中间全部是注释
+                    m_CommentSQL = None
                     for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
                         if len(SQLCommands[m_CommentPos].lstrip()) != 0:
                             # 这是一个非完全空行，有效内容从第一个字符开始
                             nLeadSpace = len(SQLCommands[m_CommentPos]) - len(SQLCommands[m_CommentPos].lstrip())
-                            m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
                             m_NewSQLWithCommentsLastPos = nLeadSpace
-                            SQLSplitResultsWithComments.append(m_CommentSQL)
                             break
                         else:
                             # 完全注释行
-                            m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                    SQLSplitResults.append("")
+                    SQLSplitResultsWithComments.append(m_CommentSQL)
                 continue
 
             # 如何本行没有任何内容，就是空行，直接结束，本语句没有任何意义
@@ -146,26 +168,35 @@ def SQLAnalyze(p_SQLCommandPlainText):
             # 如果本行每行没有包含任何关键字信息，则直接返回
             strRegexPattern = r'^(\s+)?CREATE(\s+)?|^(\s+)?SELECT(\s+)?|^(\s+)?UPDATE(\s+)?|' \
                               r'^(\s+)?DELETE(\s+)?|^(\s+)?INSERT(\s+)?|^(\s+)?__INTERNAL__(\s+)?|' \
-                              r'^(\s+)?DROP(\s+)?|^(\s+)?REPLACE(\s+)?|^(\s+)?LOAD(\s+)?'
+                              r'^(\s+)?DROP(\s+)?|^(\s+)?REPLACE(\s+)?|^(\s+)?LOAD(\s+)?|' \
+                              r'^(\s+)?MERGE(\s+)?'
             if not re.match(strRegexPattern, SQLCommands[m_nPos], re.IGNORECASE):
                 SQLSplitResults.append(SQLCommands[m_nPos])
-                m_CommentSQL = SQLCommandsWithComments[m_nPos]
+                SQLSplitResultsWithComments.append(SQLCommandsWithComments[m_nPos])
                 if m_nPos == len(SQLCommands) - 1:
                     # 如果本行就是最后一行
-                    SQLSplitResultsWithComments.append(m_CommentSQL)
+                    m_NewSQLWithCommentsLastPos = 0
                 else:
-                    # 一直找到下一个有效语句的开始
+                    # 从当前语句开始，一直找到下一个有效语句的开始，中间全部是注释
+                    m_CommentSQL = None
                     for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
                         if len(SQLCommands[m_CommentPos].lstrip()) != 0:
                             # 这是一个非完全空行，有效内容从第一个字符开始
                             nLeadSpace = len(SQLCommands[m_CommentPos]) - len(SQLCommands[m_CommentPos].lstrip())
-                            m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
                             m_NewSQLWithCommentsLastPos = nLeadSpace
-                            SQLSplitResultsWithComments.append(m_CommentSQL)
                             break
                         else:
                             # 完全注释行
-                            m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                    SQLSplitResults.append("")
+                    SQLSplitResultsWithComments.append(m_CommentSQL)
                 continue
 
             # 以下本行内一定包含了关键字，是多行SQL，或者多段SQL, 也可能是单行；结尾
@@ -176,30 +207,36 @@ def SQLAnalyze(p_SQLCommandPlainText):
                 if not re.match(strRegexPattern, SQLCommands[m_nPos], re.IGNORECASE):
                     # 这不是一个存储过程，本行可以结束了
                     SQLSplitResults.append(SQLCommands[m_nPos])
+                    SQLSplitResultsWithComments.append(SQLCommandsWithComments[m_nPos])
                     if m_nPos == len(SQLCommands) - 1:
                         # 如果本行就是最后一行
-                        SQLSplitResultsWithComments.append(m_NewSQLWithComments)
                         m_NewSQLWithCommentsLastPos = 0
-                        m_NewSQLWithComments = None
                     else:
-                        # 一直找到下一个有效语句的开始
+                        # 从当前语句开始，一直找到下一个有效语句的开始，中间全部是注释
+                        m_CommentSQL = None
                         for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
                             if len(SQLCommands[m_CommentPos].lstrip()) != 0:
                                 # 这是一个非完全空行，有效内容从第一个字符开始
                                 nLeadSpace = len(SQLCommands[m_CommentPos]) - len(SQLCommands[m_CommentPos].lstrip())
-                                m_NewSQLWithComments = \
-                                    m_NewSQLWithComments + '\n' + \
-                                    SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                                if m_CommentSQL is None:
+                                    m_CommentSQL = SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                                else:
+                                    m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][
+                                                                         0:nLeadSpace]
                                 m_NewSQLWithCommentsLastPos = nLeadSpace
-                                SQLSplitResultsWithComments.append(m_NewSQLWithComments)
-                                m_NewSQLWithComments = None
                                 break
                             else:
                                 # 完全注释行
-                                m_NewSQLWithComments = m_NewSQLWithComments + '\n' + \
-                                                       SQLCommandsWithComments[m_CommentPos]
+                                if m_CommentSQL is None:
+                                    m_CommentSQL = SQLCommandsWithComments[m_CommentPos]
+                                else:
+                                    m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                        SQLSplitResults.append("")
+                        SQLSplitResultsWithComments.append(m_CommentSQL)
+                    continue
                 else:
                     # 这是一个存储过程
+                    print("I'm here. waiting for procedure")
                     m_bInBlockSQL = True
                     m_bInMultiLineSQL = True
             else:
@@ -210,27 +247,31 @@ def SQLAnalyze(p_SQLCommandPlainText):
             if re.match(r'^/$', SQLCommands[m_nPos]):
                 SQLSplitResults.append(m_NewSQL)
                 # 注释信息包含/符号
-                m_NewSQLWithComments = m_NewSQLWithComments + '\n' + SQLCommandsWithComments[m_nPos]
+                SQLSplitResultsWithComments.append(m_NewSQLWithComments + "\n/")
                 if m_nPos == len(SQLCommands) - 1:
                     # 如果本行就是最后一行
-                    SQLSplitResultsWithComments.append(m_NewSQLWithComments)
                     m_NewSQLWithCommentsLastPos = 0
-                    m_NewSQLWithComments = None
                 else:
-                    # 一直找到下一个有效语句的开始
+                    # 从当前语句开始，一直找到下一个有效语句的开始，中间全部是注释
+                    m_CommentSQL = None
                     for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
                         if len(SQLCommands[m_CommentPos].lstrip()) != 0:
                             # 这是一个非完全空行，有效内容从第一个字符开始
                             nLeadSpace = len(SQLCommands[m_CommentPos]) - len(SQLCommands[m_CommentPos].lstrip())
-                            m_NewSQLWithComments = m_NewSQLWithComments + '\n' + SQLCommandsWithComments[m_CommentPos][
-                                                                                 0:nLeadSpace]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
                             m_NewSQLWithCommentsLastPos = nLeadSpace
-                            SQLSplitResultsWithComments.append(m_NewSQLWithComments)
-                            m_NewSQLWithComments = None
                             break
                         else:
                             # 完全注释行
-                            m_NewSQLWithComments = m_NewSQLWithComments + '\n' + SQLCommandsWithComments[m_CommentPos]
+                            if m_CommentSQL is None:
+                                m_CommentSQL = SQLCommandsWithComments[m_CommentPos]
+                            else:
+                                m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                    SQLSplitResults.append("")
+                    SQLSplitResultsWithComments.append(m_CommentSQL)
                 m_bInBlockSQL = False
                 m_bInMultiLineSQL = False
                 continue
@@ -250,29 +291,28 @@ def SQLAnalyze(p_SQLCommandPlainText):
                     if not re.match(strRegexPattern, m_NewSQL, re.IGNORECASE):
                         # 这不是一个存储过程，本行可以结束了
                         SQLSplitResults.append(m_NewSQL)
-                        if m_nPos == len(SQLCommands) - 1:
-                            # 如果本行就是最后一行
-                            SQLSplitResultsWithComments.append(m_NewSQLWithComments)
-                            m_NewSQLWithCommentsLastPos = 0
-                            m_NewSQLWithComments = None
-                        else:
-                            # 一直找到下一个有效语句的开始
-                            for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
-                                if len(SQLCommands[m_CommentPos].lstrip()) != 0:
-                                    # 这是一个非完全空行，有效内容从第一个字符开始
-                                    nLeadSpace = len(SQLCommands[m_CommentPos]) - len(
-                                        SQLCommands[m_CommentPos].lstrip())
-                                    m_NewSQLWithComments = \
-                                        m_NewSQLWithComments + '\n' + \
-                                        SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
-                                    m_NewSQLWithCommentsLastPos = nLeadSpace
-                                    SQLSplitResultsWithComments.append(m_NewSQLWithComments)
-                                    m_NewSQLWithComments = None
-                                    break
+                        SQLSplitResultsWithComments.append(m_NewSQLWithComments)
+                        # 从当前语句开始，一直找到下一个有效语句的开始，中间全部是注释
+                        m_CommentSQL = None
+                        for m_CommentPos in range(m_nPos + 1, len(SQLCommands)):
+                            if len(SQLCommands[m_CommentPos].lstrip()) != 0:
+                                # 这是一个非完全空行，有效内容从第一个字符开始
+                                nLeadSpace = len(SQLCommands[m_CommentPos]) - len(SQLCommands[m_CommentPos].lstrip())
+                                if m_CommentSQL is None:
+                                    m_CommentSQL = SQLCommandsWithComments[m_CommentPos][0:nLeadSpace]
                                 else:
-                                    # 完全注释行
-                                    m_NewSQLWithComments = m_NewSQLWithComments + '\n' + \
-                                                           SQLCommandsWithComments[m_CommentPos]
+                                    m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos][
+                                                                         0:nLeadSpace]
+                                m_NewSQLWithCommentsLastPos = nLeadSpace
+                                break
+                            else:
+                                # 完全注释行
+                                if m_CommentSQL is None:
+                                    m_CommentSQL = SQLCommandsWithComments[m_CommentPos]
+                                else:
+                                    m_CommentSQL = m_CommentSQL + '\n' + SQLCommandsWithComments[m_CommentPos]
+                        SQLSplitResults.append("")
+                        SQLSplitResultsWithComments.append(m_CommentSQL)
                         m_bInMultiLineSQL = False
                         continue
                     else:
@@ -283,6 +323,16 @@ def SQLAnalyze(p_SQLCommandPlainText):
                     # 多行语句还没有结束
                     continue
 
+    # 如果当前语句没有结束，但是文件结束了，送最后一段到SQL引擎中
+    if m_bInMultiLineSQL:
+        SQLSplitResults.append(m_NewSQL)
+        SQLSplitResultsWithComments.append(m_NewSQLWithComments)
+
+    # 去掉SQL语句中的最后一个；
+    for m_nPos in range(0, len(SQLSplitResults)):
+        if SQLSplitResults[m_nPos][-1:] == ';':
+            SQLSplitResults[m_nPos] = SQLSplitResults[m_nPos][:-1]
+
     # 去掉注释信息中的最后一个回车换行符
     for m_nPos in range(0, len(SQLSplitResultsWithComments)):
         if SQLSplitResultsWithComments[m_nPos][-1:] == '\n':
